
package com.mycompany.plugins.user;

import android.content.*;
import android.util.*;
import android.view.*;
import android.view.ViewGroup.*;
import android.view.View.*;
import android.widget.*;
import java.util.*;

import org.opencv.android.*;
import org.opencv.core.*;
import org.opencv.android.CameraBridgeViewBase.*;
import org.opencv.imgproc.Imgproc;

//TextView wrapper for calling via javascript.
public class CvView extends JavaCameraView implements CvCameraViewListener2, OnTouchListener
{		
	public static String TAG = "MyPlugin";	 
	private Context m_ctx;
	private Mat mRgba;
	//private Mat mGray;

	private enum Mode { CANNY, BLOB, SOBEL }
	private Mode m_mode = Mode.CANNY;

	//Shared.
	private Mat mIntermediateMat;
	private boolean m_colourTrans = true;
	private Mat m_mColourKernel = null;

	//Canny edge detector.
	private double m_cannyThresh1 = 80;
	private double m_cannyThresh2 = 100;

	//Sobel edge detector
	private int m_sobelDx = 1;
	private int m_sobelDy = 1;
    
	//Colour Blob.
	private Object m_blobLock = new Object();
	//List<MatOfPoint> m_blobs = new ArrayList<MatOfPoint>();
	private String m_blobs;
	private boolean              mIsColorSelected = false;
    private Scalar               mBlobColorRgba;
    private Scalar               mBlobColorHsv;
    private ColorBlobDetector    mDetector;
    private Mat                  mSpectrum;
    private Size                 SPECTRUM_SIZE;
    private Scalar               CONTOUR_COLOR;


	public CvView( Context context, int width, int height ) 
	{ 
		super( context, -1 );
		m_ctx = context;
		Log.i(TAG, "Creating CvView");
		
		//mOpenCvCameraView = (CameraBridgeViewBase) new JavaCameraView(context, -1);
		//mOpenCvCameraView.setVisibility(SurfaceView.VISIBLE);
		setCvCameraViewListener( this );
		setOnTouchListener( this );

		//LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams( 
		//			LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT);
		LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams( width, height );
		setLayoutParams( lp );
	}

	//Set the current IP mode.
	public void SetMode( String modes )
	{
		//Todo: use bitmask and allow more than one at once.
		modes = modes.toLowerCase(Locale.ENGLISH);
		if( modes.contains("blob") ) m_mode = Mode.BLOB;
		else if( modes.contains("canny") ) m_mode = Mode.CANNY;
		else if( modes.contains("sobel") ) m_mode = Mode.SOBEL;
	}

	//Set canny options.
	public void SetCanny( double thresh1, double thresh2 ) 
	{
		m_cannyThresh1 = thresh1; 
		m_cannyThresh2 = thresh2;
	}

	//Set sobel options.
	public void SetSobel( int dx, int dy )
	{
		m_sobelDx = dx;
		m_sobelDy = dy;
	}

	//Set the colour matrix to adjust output colour.
	public void SetColorMatrix( String values )
	{
		//Extract the colour matrix vals.
		String[] ss = values.split(","); 
		float[] ff = new float[ss.length]; 
		for( int i=0; i<ss.length; i++ ) 
			ff[i] = Float.parseFloat(ss[i]); 

		//Fill colour kernel.
        m_mColourKernel = new Mat(4, 4, CvType.CV_32F);
        m_mColourKernel.put( 0, 0, ff[0], ff[1], ff[2], ff[3] );		//R
        m_mColourKernel.put( 1, 0, ff[4], ff[5], ff[6], ff[7] );		//G
        m_mColourKernel.put( 2, 0, ff[8], ff[9], ff[10], ff[11] );		//B
        m_mColourKernel.put( 3, 0, ff[12], ff[13], ff[14], ff[15] );	//A
	}

	//Get the blob list.
	public String GetBlobs()
	{
		Log.d( TAG, "GetBlobs" );
		//synchronized( m_blobLock ) {
			return m_blobs; //.clone();
		//}
	}

	private String BuildBlobString( List<MatOfPoint> blobs )
	{
		StringBuilder s = new StringBuilder("[");

		Iterator<MatOfPoint> each = blobs.iterator();
		while( each.hasNext() )
		{
			if( s.length() > 1 ) s.append( "," );
			s.append( "[" );

			MatOfPoint mat = each.next();
			Point[] list = mat.toArray();
			for( int i=0; i < list.length; i++ ) 
			{
				if( i>0 ) s.append( "," );
				s.append( "{" );
				s.append( "x:" + list[i].x );
				s.append( ",y:" + list[i].y );
				s.append( "}" );
			}
			s.append( "]" );
		}
		s.append( "]" );
		return s.toString();
	}

	@Override 
	public void onCameraViewStarted(int width, int height) 
	{
		if( m_mode==Mode.BLOB )
		{
			mRgba = new Mat(height, width, CvType.CV_8UC4);
			mDetector = new ColorBlobDetector();
			mSpectrum = new Mat();
			mBlobColorRgba = new Scalar(255);
			mBlobColorHsv = new Scalar(255);
			SPECTRUM_SIZE = new Size(200, 64);
			CONTOUR_COLOR = new Scalar(255,0,0,255);
		}
		else if( m_mode==Mode.CANNY )
		{
			mRgba = new Mat(height, width, CvType.CV_8UC4);
			mIntermediateMat = new Mat(height, width, CvType.CV_8UC4);
			//mGray = new Mat(height, width, CvType.CV_8UC1);
		}
		else if( m_mode==Mode.SOBEL )
		{
			mRgba = new Mat(height, width, CvType.CV_8UC4);
			mIntermediateMat = new Mat(height, width, CvType.CV_8UC4);
			//mGray = new Mat(height, width, CvType.CV_8UC1);
		}
	}

	public void onCameraViewStopped() 
	{
		if( m_mode==Mode.BLOB )
		{
			mRgba.release();
			mSpectrum.release();
		}
		else if( m_mode==Mode.CANNY )
		{
			mRgba.release();
			//mGray.release();
			mIntermediateMat.release();
		}
		else if( m_mode==Mode.SOBEL )
		{
			mRgba.release();
			mIntermediateMat.release();
		}
	}

	public Mat onCameraFrame(CvCameraViewFrame inputFrame) 
	{
		mRgba = inputFrame.rgba();

		//Log.i(TAG, "onCameraFrame");
		if( m_mode==Mode.BLOB )
		{
			if (mIsColorSelected) 
			{
				//Detect countours.
				mDetector.process(mRgba);
				List<MatOfPoint> contours = mDetector.getContours();
				Log.e(TAG, "Contours count: " + contours.size());
				Imgproc.drawContours(mRgba, contours, -1, CONTOUR_COLOR);

				//Save contours for DS to pick up.
				//synchronized( m_blobLock ) {
					m_blobs = BuildBlobString( contours );
				//}

				//Draw colour info on screen.
				Mat colorLabel = mRgba.submat(4, 68, 4, 68);
				colorLabel.setTo(mBlobColorRgba);
				Mat spectrumLabel = mRgba.submat(4, 4 + mSpectrum.rows(), 70, 70 + mSpectrum.cols());
				mSpectrum.copyTo(spectrumLabel);
			}
		}
		else if( m_mode==Mode.CANNY ) //Fast
		{
			Imgproc.Canny(inputFrame.gray(), mIntermediateMat, m_cannyThresh1, m_cannyThresh2 ); //80,100
			//Imgproc.cvtColor(mIntermediateMat, mRgba, Imgproc.COLOR_GRAY2BGRA, 4);
			Core.convertScaleAbs( mIntermediateMat, mIntermediateMat, 10, 0 );
			Imgproc.cvtColor(mIntermediateMat, mRgba, Imgproc.COLOR_GRAY2BGRA, 4);
		}
		else if( m_mode==Mode.SOBEL ) //Slow
		{
            Imgproc.Sobel( inputFrame.gray(), mIntermediateMat, CvType.CV_8U, m_sobelDx, m_sobelDy );
            Core.convertScaleAbs( mIntermediateMat, mIntermediateMat, 10, 0 );
            Imgproc.cvtColor( mIntermediateMat, mRgba, Imgproc.COLOR_GRAY2BGRA, 4 );
		}

		if( m_colourTrans )
		{
            if( m_mColourKernel != null) 
				Core.transform( mRgba, mRgba, m_mColourKernel );

			/*
			//incredibly slow!
			//int len = mRgba.cols() *  mRgba.rows();
			//byte[] rgbaOut = { (byte)0,(byte)128,(byte)0,(byte)255 };
			byte[] rgbaVal = {0,0,0,0};
			for( int r=0; r<mRgba.rows(); r++ ) 
				for( int c=0; c<mRgba.cols(); c++ )
				{
					mRgba.get( r, c, rgbaVal );
					rgbaVal[1] = (byte)255;
					mRgba.put( r, c, rgbaVal );
				}
				*/

		}

		return mRgba;
	}		
	
	public boolean onTouch(View v, MotionEvent event) 
	{
        int cols = mRgba.cols();
        int rows = mRgba.rows();

        int xOffset = (getWidth() - cols) / 2;
        int yOffset = (getHeight() - rows) / 2;

        int x = (int)event.getX() - xOffset;
        int y = (int)event.getY() - yOffset;

        Log.i(TAG, "Touch image coordinates: (" + x + ", " + y + ")");
        if ((x < 0) || (y < 0) || (x > cols) || (y > rows)) return false;

		if( m_mode==Mode.BLOB )
		{
			Rect touchedRect = new Rect();
			touchedRect.x = (x>4) ? x-4 : 0;
			touchedRect.y = (y>4) ? y-4 : 0;
			touchedRect.width = (x+4 < cols) ? x + 4 - touchedRect.x : cols - touchedRect.x;
			touchedRect.height = (y+4 < rows) ? y + 4 - touchedRect.y : rows - touchedRect.y;

			Mat touchedRegionRgba = mRgba.submat(touchedRect);
			Mat touchedRegionHsv = new Mat();
			Imgproc.cvtColor(touchedRegionRgba, touchedRegionHsv, Imgproc.COLOR_RGB2HSV_FULL);

			// Calculate average color of touched region
			mBlobColorHsv = Core.sumElems(touchedRegionHsv);
			int pointCount = touchedRect.width*touchedRect.height;
			for (int i = 0; i < mBlobColorHsv.val.length; i++)
				mBlobColorHsv.val[i] /= pointCount;

			mBlobColorRgba = converScalarHsv2Rgba(mBlobColorHsv);

			Log.i(TAG, "Touched rgba color: (" + mBlobColorRgba.val[0] + ", " + mBlobColorRgba.val[1] +
					", " + mBlobColorRgba.val[2] + ", " + mBlobColorRgba.val[3] + ")");

			mDetector.setHsvColor(mBlobColorHsv);
			Imgproc.resize(mDetector.getSpectrum(), mSpectrum, SPECTRUM_SIZE);
			mIsColorSelected = true;

			touchedRegionRgba.release();
			touchedRegionHsv.release();
		}

        return false; // don't need subsequent touch events
    }

	 private Scalar converScalarHsv2Rgba(Scalar hsvColor) 
	 {
        Mat pointMatRgba = new Mat();
        Mat pointMatHsv = new Mat(1, 1, CvType.CV_8UC3, hsvColor);
        Imgproc.cvtColor(pointMatHsv, pointMatRgba, Imgproc.COLOR_HSV2RGB_FULL, 4);
        return new Scalar(pointMatRgba.get(0, 0));
    }

	
 }


/* todo
mResolutionList = mOpenCvCameraView.getResolutionList();
    mResolutionMenuItems = new MenuItem[mResolutionList.size()];

    ListIterator<Camera.Size> resolutionItr = mResolutionList.listIterator();
    idx = 0;
    while(resolutionItr.hasNext()) {
        Camera.Size element = resolutionItr.next();
        mResolutionMenuItems[idx] = mResolutionMenu.add(1, idx, Menu.NONE,
                Integer.valueOf((int) element.width).toString() + "x" + Integer.valueOf((int) element.height).toString());
        idx++;
     }

    return true;
}

public boolean onOptionsItemSelected(MenuItem item) {
    Log.e(TAG, "called onOptionsItemSelected; selected item: " + item);
   if (item.getGroupId() == 1)
    {
        int id = item.getItemId();
        Camera.Size resolution = mResolutionList.get(id);
        mOpenCvCameraView.setResolution(resolution);
        resolution = mOpenCvCameraView.getResolution();
        Log.e("test","test");
		*/