
/*
	DroidScript Plugin class.
	(This is where you put your plugin code)
*/

package com.mycompany.plugins.user;

import android.os.*;
import android.content.*;
import android.util.Log;
import java.util.*;
import android.view.*;
import android.widget.*;
import android.graphics.*;
import android.graphics.drawable.*;
import java.io.*;
import java.lang.reflect.*;

import org.opencv.android.*;
import org.opencv.core.*;
import org.opencv.android.CameraBridgeViewBase.*;

public class MyPlugin
{
	public static String TAG = "MyPlugin";	
	public static float VERSION = 1.0f;	
	private Method m_callscript;
	private Object m_parent;
	private Context m_ctx;

	private BaseLoaderCallback mLoaderCallback;
	private CvView mOpenCvCameraView;

	//Script callbacks.
	private String m_OnMyReply;

	//Contruct plugin.
	public MyPlugin()
	{
		Log.d( TAG, "Creating plugin object");
	}

	//Initialise plugin.
	public void Init( Context ctx, Object parent )
	{
		try {
			Log.d( TAG, "Initialising plugin object");

			//Save context and reference to parent (AndroidScript).
			m_ctx = ctx;
			m_parent = parent;

			//Use reflection to get 'CallScript' method
			Log.d( TAG, "Getting CallScript method");
			m_callscript = parent.getClass().getMethod( "CallScript", Bundle.class );

			//Handle OpenCV Manager callbacks.
			SetManagerCallback( ctx );
		 } 
		 catch (Exception e) {
			   Log.e( TAG, "Failed to Initialise plugin!", e );
		 }
	}

	//Release plugin resources.
	public void Release()
	{
	}

	//Get a plugin resource file.
	//Test for resource fetching.
	//	File f = GetRes( "fred.txt" );
	//	Log.d( TAG, "res name = " + f.getPath() );
	public File GetRes( String path )
	{
		File privDir = m_ctx.getDir( "Plugins", 0 );
		String privPath = privDir.getAbsolutePath();
		File f = new File( privDir + "/"+getClass().getSimpleName().toLowerCase()+"/res/" + path );
		return f;
	}

	//Call a function in the user's script.
	private void CallScript( Bundle b )
	{
		try {
			m_callscript.invoke( m_parent, b );
		} 
		catch (Exception e) {
			Log.e( TAG, "Failed to call script function!", e );
		}
	}

	//Handle older style commands from DroidScript.
	public String CallPlugin( Bundle b )
	{
		return CallPlugin( b, null );
	}

	//Handle commands from DroidScript.
	public String CallPlugin( Bundle b, Object obj )
	{
		//Extract command.
		String cmd = b.getString("cmd");
	
		//Process commands.
		String ret = null;
		try 
		{
			if( cmd.equals("GetPixelData") )
			{
				ret = GetPixelData( b.getString("p1"), (int)b.getFloat("p2"),  
					(int)b.getFloat("p3"), (int)b.getFloat("p4"), (int)b.getFloat("p5"), (int)b.getFloat("p6") );
			}
			else if( cmd.equals("GetVersion") ) 
				return GetVersion( b );
			else if( cmd.equals("GetBlobs") )
				return mOpenCvCameraView.GetBlobs();

			else if( cmd.equals("MyFunc") ) 
				MyFunc( b );
			else if( cmd.equals("SetOnMyReply") ) 
				SetOnMyReply( b );
			
			else if( cmd.equals("SetMode") ) 
			{
				mOpenCvCameraView.disableView();
				mOpenCvCameraView.SetMode( b.getString("p1") );
				mOpenCvCameraView.enableView();
			}
			else if( cmd.equals("SetCanny") ){
				mOpenCvCameraView.SetCanny( b.getFloat("p1"), b.getFloat("p2") );
			}
			else if( cmd.equals("SetSobel") ){
				mOpenCvCameraView.SetSobel( (int)b.getFloat("p1"), (int)b.getFloat("p2") );
			}
			else if( cmd.equals("SetColorMatrix") ) {
				mOpenCvCameraView.SetColorMatrix( b.getString("p1") );
			}
			
		} 
		catch (Exception e) {
		   Log.e( TAG, "Plugin command failed!", e);
		}
		return ret;
	}


	/* todo
	Bitmap cmapBitmap = Bitmap.createBitmap(n, 1, Bitmap.Config.ARGB_8888);
	Utils.matToBitmap(cmap, cmapBitmap, false);
	setImageBitmap(cmapBitmap

	*/

	//Handle creating object from DroidScript.
	public Object CreateObject( Bundle b )
	{
		//Extract object type.
		String type = b.getString("type");
	
		//Process commands.
		Object ret = null;
		try {
			if( type.equals("MyButton") ) 
				return CreateMyButton( b );
		} 
		catch (Exception e) {
		   Log.e( TAG, "Plugin command failed!", e);
		}
		return ret;
	}


	//Handle the GetVersion command.
	private String GetVersion( Bundle b )
	{
		Log.d( TAG, "Got GetVersion" );
		return Float.toString( VERSION );
	}
	
	//Handle the 'SetOnMyReply' command.
	private void SetOnMyReply( Bundle b )
	{
		Log.d( TAG, "Got SetOnMyReply" );
		m_OnMyReply = b.getString("p1");
	}

	//Handle the 'MyFunc' command.
	private void MyFunc( Bundle b )
	{
		Log.d( TAG, "Got MyFunc" );

		//Extract params from 'MyFunc' command.
		String txt = b.getString("p1");
		float num = b.getFloat("p2");
		boolean boo = b.getBoolean("p3");
	
		//If 'OnReply' callback is set.
		if( m_OnMyReply!=null )
		{
			//Fire callback in script.
			b = new Bundle();
			b.putString( "cmd", m_OnMyReply );
			b.putString( "p1", txt + " world" );
			b.putFloat( "p2", num+20 );
			b.putBoolean( "p3", !boo );
			CallScript( b );
		}
	}

	//Handle 'CreateMyButton' command.
	private Object CreateMyButton( Bundle b )
	{
		//Extract params.
		String txt = b.getString("p1");
		float width = b.getFloat("p2");
		float height = b.getFloat("p3");

		//Create custom button.
		mOpenCvCameraView = new CvView( m_ctx, (int)width, (int)height );

		//Init OpenCV library (via Manager App).
		Log.d(TAG, "Using OpenCV Manager for initialization...");
        OpenCVLoader.initAsync( OpenCVLoader.OPENCV_VERSION_3_0_0, m_ctx, mLoaderCallback );

		return mOpenCvCameraView;
	}

	//Get image data to base64 string (rgba8888 html canvas compatible).
	public String GetPixelData( String options, int left, int top, int width, int height, int rotation )
	{
		/*if( options.toLowerCase().contains("luma") ) 
		{
			//Useful for low res image processing (don't need to use 'UseBitmap' option).
			//(Resizing not possible, just returns Y (luma) part of YUV.
			synchronized ( m_previewFrameDataLock ) {
				return Base64.encodeToString( m_previewFrameData, Base64.NO_WRAP );
			}
		}
		else */
		Bitmap bmp = mOpenCvCameraView.getBitmap();
		Object lock = mOpenCvCameraView.getBitmapLock();
		return Utils.GetPixelData( bmp, lock, options, left, top, width, height, false, rotation );
	}

	//Handle OpenCV Manager callbacks.
	private void SetManagerCallback( Context ctx )
	{
		mLoaderCallback = new BaseLoaderCallback(ctx) 
		 {
			@Override
			public void onManagerConnected(int status) 
			{
				switch (status) 
				{
					case LoaderCallbackInterface.SUCCESS:
					{
						Log.i(TAG, "OpenCV loaded successfully");
						mOpenCvCameraView.enableView();
					} break;
					default:
					{
						super.onManagerConnected(status);
					} break;
				}
			}
		};
	}

} 

