

package com.mycompany.plugins.user;

import android.util.*;
import android.graphics.*;
import android.graphics.drawable.*;
import java.text.*;
import java.io.*;
import java.util.*;

public class Utils 
{
	static class Size { int width; int height;  public Size( int w, int h ) { width=w; height=h; } }
	private static int[] s_pixBuffer;
	private static int s_pixBufSize = 0;

	//Get bitmap data to base64 string (rgba8888 html canvas compatible).
	public static String GetPixelData( Bitmap bmpIn, Object bmpLock, String options, 
					int left, int top, int width, int height, boolean isCamera, int rotation )
	{
		if( bmpIn == null ) return null;
		options = options.toLowerCase(Locale.ENGLISH);

		//Get chosen part of video frame.
		synchronized( bmpLock ) 
		{
			//Rescale image if required.
			boolean recycleBmpNew = false;
			Bitmap bmpNew = bmpIn;
			Size size = GetResolution( options );
			boolean rot90 = ( Math.abs(rotation)==90 );
			if( size.width > 0 && size.height > 0 || rot90 ) 
			{
				if( rot90 ) bmpNew = Bitmap.createScaledBitmap( bmpIn, size.height, size.width, true );
				else bmpNew = Bitmap.createScaledBitmap( bmpIn, size.width, size.height, true );
				recycleBmpNew = true;
			}

			//Rotate image if required.
			Bitmap bmpRot = bmpNew;
			boolean recycleBmpRot = false;
			if( rotation != 0 ) 
			{
				//Swap aspect ratio of image.
				//Bitmap bmpScale = Bitmap.createScaledBitmap( bmpNew, bmpNew.getHeight(), bmpNew.getWidth(), true );
				//if( recycleBmpNew ) bmpNew.recycle();

				//Rotate image
				Matrix matrix = new Matrix();
				matrix.postRotate( rotation );
				bmpRot = Bitmap.createBitmap( bmpNew, 0, 0, bmpNew.getWidth(), bmpNew.getHeight(), matrix, true );
				//bmpScale.recycle();
				recycleBmpRot = true;
			}
			
			//Check required image window.
			if( width==0 ) width = bmpRot.getWidth();
			if( height==0 ) height = bmpRot.getHeight();

			//Allocate pixel buffer if changed size.
			if( s_pixBufSize != width * height ) {
				s_pixBufSize = width * height;
				s_pixBuffer = new int[ s_pixBufSize ];
			}

			//Grab pixel data.
			bmpRot.getPixels( s_pixBuffer, 0, width, left, top, width, height );
			if( recycleBmpRot ) bmpRot.recycle();

			//Apply mask if required.
			if( options.contains("green") ) Utils.ColorMask( s_pixBuffer, 0xff00ff00 );
		}

		if( options.indexOf("rawbase64")>-1 ) 
		{
			if( options.contains("grayscale") ) {
				return Base64.encodeToString( Utils.int2byte8(s_pixBuffer), Base64.NO_WRAP );
			}
			else {
				//Swap red and blue components if requred.
				if( !isCamera ) Utils.ABGRtoARGB( s_pixBuffer );
				return Base64.encodeToString( Utils.int2byte8888(s_pixBuffer), Base64.NO_WRAP );
			}
		}
		else if( options.indexOf("pngbase64")>-1 || options.indexOf("jpgbase64")>-1 ) 
		{
			//Swap red and blue components if requred.
			if( isCamera ) Utils.ABGRtoARGB( s_pixBuffer );

			//Put into a new bitmap of required size.
			Bitmap bmp = Bitmap.createBitmap( width, height, Bitmap.Config.ARGB_8888 );
			bmp.setPixels( s_pixBuffer, 0, width, 0, 0, width, height );

			//Encode to PNG base64 and recycle bitmap.
			ByteArrayOutputStream baos = new ByteArrayOutputStream();  
			if( options.indexOf("pngbase64")>-1 ) bmp.compress( Bitmap.CompressFormat.PNG, 100, baos );
			else bmp.compress( Bitmap.CompressFormat.JPEG, 100, baos );
			bmp.recycle();
			byte[] b = baos.toByteArray();
			return Base64.encodeToString( b, Base64.NO_WRAP );
		}
		else if( options.indexOf("json")>-1 ) //<- 3x slower than rawbase64.
		{
			byte[] bytes = Utils.int2byte8888( s_pixBuffer );
			int len = bytes.length;

			StringBuilder s = new StringBuilder( len*2 );
			s.append( "[" );
			for( int i=0; i<len; i++ ) {
				if( i>0 ) s.append( "," );
				s.append( bytes[i] & 0xff );
			}
			s.append( "]" );
			return s.toString();
		}
		return null;
	}

	
	//Get image resolution from standard names.
	public static Size GetResolution( String options )
	{
		int w=0, h=0;
		String ops = options.toUpperCase();
		if( ops.contains("CIF") ) { w = 352; h = 288; }
		else if( ops.contains("QQVGA") ) { w = 160; h = 120; }
		else if( ops.contains("QVGA") ) { w = 320; h = 240; }
		else if( ops.contains("SVGA") ) { w = 800; h = 600; }
		else if( ops.contains("VGA") ) { w = 640; h = 480; }
		else if( ops.contains("UXGA") ) { w = 1600; h = 1200; }
		else if( ops.contains("XGA") ) { w = 1024; h = 768; }
		
		return new Size( w,h );
	}

	//Swap ABGR8888 to ARGB8888.
	public static void ColorMask( int[] buffer, int mask )
	{
		int bufSize = buffer.length;
		for( int i=0; i<bufSize; i++ ) 
		{
			buffer[i] = buffer[i] & mask & 0xFFFFFFFF;
		}
	}

	//Swap ABGR8888 to ARGB8888.
	public static void ABGRtoARGB( int[] buffer )
	{
		int alpha, red, green, blue;
		int bufSize = buffer.length;
		for( int i=0; i<bufSize; i++ ) 
		{
			alpha = (buffer[i] >> 24) & 0xFF;
			red = (buffer[i] >> 16) & 0xFF;
			green = (buffer[i] >> 8) & 0xFF;
			blue = (buffer[i] >> 0) & 0xFF;
			buffer[i] = (alpha << 24) | (blue << 16) | (green << 8) | red;
		}
	}

	//Convert integer pixel array to 32 bit 8888 byte array.
	public static byte[] int2byte8888( int[] src ) 
	{
		int srcLength = src.length;
		byte[]dst = new byte[srcLength << 2];
	    
		for (int i=0; i<srcLength; i++) 
		{
			int x = src[i];
			int j = i << 2;
			dst[j++] = (byte) ((x >>> 0) & 0xff);           
			dst[j++] = (byte) ((x >>> 8) & 0xff);
			dst[j++] = (byte) ((x >>> 16) & 0xff);
			dst[j++] = (byte) ((x >>> 24) & 0xff);
		}
		return dst;
	}

	//Convert integer pixel array to 8 bit (greyscale) array.
	public static byte[] int2byte8( int[] src ) 
	{
		int srcLength = src.length;
		byte[] dst = new byte[srcLength];
	    
		int p, red, green, blue;
		for( int i=0; i<srcLength; i++ ) 
		{
			p = src[i];
			red = p & 0xff;           
			green = (p >>> 8) & 0xff;
			blue = (p >>> 16) & 0xff;
			dst[i] = (byte) (( (red + green + blue)/3) & 0xff);
		}
		return dst;
	}

}

